use axum::{
	RequestPartsExt,
	extract::{FromRequestParts, State},
	http::{StatusCode, header::HeaderMap, request::Parts},
};
use snafu::prelude::*;
use url::Url;

use crate::{
    error::*,
    state::{RoutableAppState, LoggedInUser},
    state::sessions::*,
};

// TODO: Implement as a typed header
pub struct OriginalURI(Url);

#[async_trait]
impl FromRequestParts<RoutableAppState> for OriginalURI {
    type Rejection = Error;

    async fn from_request_parts(parts: &mut Parts, state: &RoutableAppState) -> Result<Self, Self::Rejection> {
		let headers: HeaderMap = parts.extract().await.unwrap();
		if let Some(uri) = headers.get("X-Original-URI") {
			// TODO: error
			Ok(
				OriginalURI(Url::parse(uri.to_str().unwrap()).unwrap())
			)
	    } else {
			// TODO: error
			panic!()
		}
	}
}

impl AsRef<Url> for OriginalURI {
    fn as_ref(&self) -> &Url {
        &self.0
    }
}


#[debug_handler]
pub async fn route(user: Option<LoggedInUser>, uri: Option<OriginalURI>, state: State<RoutableAppState>) -> StatusCode {
    if uri.is_none() {
        panic!();
    }

    let uri = uri.unwrap();
    let username = user.map(|u| u.username().clone());

    match state.permissions.ssowat_config() {
        Ok(conf) => {
            if conf.user_has_permission_for_uri(username.as_ref(), uri.as_ref()) {
                StatusCode::OK
            } else {
                StatusCode::FORBIDDEN
            }
        }, Err(e) => {
            panic!()
        }
    }
}
